import json
import logging
import pytest
import webtest

from glibs.www import log_extra_info_on_4xx5xx_errors

# App behavior
#
# Single endpoint /open-door
#
# Accepts GET (?door=X&passprahse=Y) or POST {"door": X, "passphrase": Y}
#
# Returns HTTP 400 if door is not geekie, using JSON
# Returns HTTP 403 if passphrase is not sempre em frente, not using JSON


def pyramid_app():
    pyramid = pytest.importorskip("pyramid")
    import pyramid.config
    import pyramid.httpexceptions

    def get(request):
        return get_or_post(request, **request.GET)

    def post(request):
        if request.headers["Content-Type"] != "application/json":
            request.response.status_int = 415
            return {"Content-Type": "expected application/json"}

        return get_or_post(request, **request.json_body)

    def get_or_post(request, door, passphrase, **extra):
        if door != "geekie":
            request.response.status_int = 404
            return {"door": "does not exist"}

        if passphrase != "sempreemfrente":
            raise pyramid.httpexceptions.HTTPForbidden()

        return {"open": True}

    config = pyramid.config.Configurator()
    config.add_route("open one door", pattern="/open-door")
    config.add_view(
        get, route_name="open one door", renderer="json", request_method="GET"
    )
    config.add_view(
        post, route_name="open one door", renderer="json", request_method="POST"
    )
    log_extra_info_on_4xx5xx_errors.bind_pyramid(config, log_maxsize=1000)
    return webtest.TestApp(config.make_wsgi_app())


def flask_app():
    flask = pytest.importorskip("flask")

    app = flask.Flask(__name__)

    def json_response(status, data):
        return flask.make_response(
            (json.dumps(data), status, {"Content-Type": "application/json"})
        )

    @app.route("/open-door", methods=["GET"])
    def get():
        return get_or_post(**flask.request.args.to_dict())

    @app.route("/open-door", methods=["POST"])
    def post():
        if flask.request.headers["Content-Type"] != "application/json":
            return json_response(415, {"Content-Type": "expected application/json"})

        return get_or_post(**flask.request.json)

    def get_or_post(door, passphrase, **extra):
        if door != "geekie":
            return json_response(404, {"door": "does not exist"})

        if passphrase != "sempreemfrente":
            flask.abort(403)

        return json_response(200, {"open": True})

    log_extra_info_on_4xx5xx_errors.bind_flask(app, log_maxsize=1000)
    return webtest.TestApp(app)


@pytest.fixture(params=[flask_app, pyramid_app])
def app(request, caplog):
    caplog.set_level(logging.INFO)
    return request.param()


def test_nothing_logged_when_status_code_200(app, caplog):
    app.get("/open-door", params={"door": "geekie", "passphrase": "sempreemfrente"})
    app.post(
        "/open-door",
        params=json.dumps({"door": "geekie", "passphrase": "sempreemfrente"}),
        headers={"Content-Type": "application/json"},
    )
    assert caplog.records == []


def test_response_logged_when_status_code_4xx_and_json_response(app, caplog):
    response = app.get(
        "/open-door",
        params={"door": "heaven", "passphrase": "abc123"},
        expect_errors=True,
    )
    assert response.status_code == 404
    assert '"door": "does not exist"' in caplog.text


def test_request_logged_when_status_code_4xx_and_json_request(app, caplog):
    response = app.post(
        "/open-door",
        params=json.dumps({"door": "heaven", "passphrase": "abc123"}),
        headers={"Content-Type": "application/json"},
        expect_errors=True,
    )
    assert response.status_code == 404
    assert '"door": "heaven"' in caplog.text


def test_no_crash_when_status_code_4xx_and_not_json_response(app):
    # Our forbidden handler intentionally "forgets" to return a JSON response
    response = app.get(
        "/open-door",
        params={"door": "geekie", "passphrase": "wrong"},
        expect_errors=True,
    )
    assert response.status_code == 403


def test_no_crash_when_status_code_4xx_and_not_json_request(app):
    response = app.post(
        "/open-door",
        params="not json",
        headers={"Content-Type": "text/plain"},
        expect_errors=True,
    )
    assert response.status_code == 415


def test_huge_requests_are_redacted(app, caplog):
    huge_request = json.dumps(
        dict(
            {"field_{}".format(i): "value_{}".format(i) for i in range(10000)},
            door="geekie",
            passphrase="wrong",
        )
    )
    response = app.post(
        "/open-door",
        params=huge_request,
        headers={"Content-Type": "application/json"},
        expect_errors=True,
    )
    assert response.status_code == 403  # KeyError("door")
    assert len(huge_request) > len(caplog.text)



import gevent
import pytest
import sys
import time
import webtest

from glibs.www import timeout


class AnotherException(Exception):
    pass


def pyramid_app():
    pyramid = pytest.importorskip("pyramid")

    config = pyramid.config.Configurator()
    config.registry.settings["gevent_timeout"] = "1"
    config.registry.settings["custom_timeout_by_endpoint"] = {
        "/test/custom/timeout/ok": 48
    }

    def view(request):
        if request.matchdict.get("op") == "timeout":
            gevent.sleep(40)
        elif request.matchdict.get("op") == "exception":
            raise AnotherException
        elif request.matchdict.get("op") == "python-timeout":
            now = time.time()
            while time.time() - now < 1.1:
                pass
        return "ok"

    def viewCustomTimeout(request):
        if request.matchdict.get("op") == "ok":
            gevent.sleep(40)
        elif request.matchdict.get("op") == "timeout":
            gevent.sleep(48)
        return "ok"

    config.add_route("test route", pattern="/test/{op}")
    config.add_route("test custom timeout route", pattern="/test/custom/timeout/{op}")
    config.add_view(view, route_name="test route", renderer="string")
    config.add_view(
        viewCustomTimeout, route_name="test custom timeout route", renderer="string"
    )

    timeout.bind_pyramid(config)

    return webtest.TestApp(config.make_wsgi_app())


def flask_app():
    flask = pytest.importorskip("flask")

    app = flask.Flask(__name__)
    app.config["gevent_timeout"] = "1"
    app.testing = True

    @app.route("/test/<op>")
    def view(op):
        if op == "timeout":
            gevent.sleep(40)
        elif op == "exception":
            raise AnotherException()
        elif op == "python-timeout":
            now = time.time()
            while time.time() - now < 1.1:
                pass
        return "ok"

    @app.route("/test/custom/timeout/<op>")
    def viewCustomTimeout(op):
        if op == "ok":
            return "ok"
        elif op == "timeout":
            now = time.time()
            while time.time() - now < 1.1:
                pass
        return "ok"

    timeout.bind_flask(app)

    return webtest.TestApp(app)


@pytest.fixture(params=[flask_app, pyramid_app])
def app(request):
    return request.param()


def test_ok(app):
    response = app.get("/test/ok")
    assert response.status_code == 200


def test_ok_with_custom_timeout(app):
    response = app.get("/test/custom/timeout/ok")
    assert response.status_code == 200


def test_timeout_on_gevent(app):
    with pytest.raises(timeout.DeadlineExceededError):
        app.get("/test/timeout")


def test_timeout_on_custom_timeout(app):
    with pytest.raises(timeout.DeadlineExceededError):
        app.get("/test/custom/timeout/timeout")


def test_timeout_on_python(app):
    with pytest.raises(timeout.DeadlineExceededError):
        app.get("/test/python-timeout")


def test_timeout_on_python_and_no_module_gevent(app, monkeypatch):
    monkeypatch.setitem(sys.modules, "gevent", None)
    with pytest.raises(timeout.DeadlineExceededError):
        app.get("/test/python-timeout")


def test_keeps_exception(app):
    with pytest.raises(AnotherException):
        app.get("/test/exception")



# coding: utf-8
import json
import logging
import pytest
import re
import webtest

from glibs.www import request_logger


class ViewError(Exception):
    pass


def flask_app(get_user_id=lambda: "john"):
    flask = pytest.importorskip("flask")
    app = flask.Flask(__name__)

    @app.route("/test")
    def test_route():
        return "ok"

    @app.route("/static/<path>")
    def static_route(path):
        return "static"

    @app.route("/error")
    def error_route():
        raise ViewError()

    request_logger.bind_flask(app, get_user_id)
    return webtest.TestApp(app)


def pyramid_app(get_user_id=lambda args: "john"):
    pyramid = pytest.importorskip("pyramid")

    import pyramid.config

    config = pyramid.config.Configurator()
    config.add_route("test route", pattern="/test")
    config.add_route("error route", pattern="/error")
    config.add_route("static route", pattern="/static/{path}")
    config.add_view(lambda r: "ok", route_name="test route", renderer="string")
    config.add_view(lambda r: "static", route_name="static route", renderer="string")

    def error_view(request):
        raise ViewError()

    config.add_view(error_view, route_name="error route", renderer="string")

    request_logger.bind_pyramid(config, get_user_id)

    return webtest.TestApp(config.make_wsgi_app())


@pytest.fixture(params=[flask_app, pyramid_app])
def app(request, caplog):
    caplog.set_level(logging.INFO)
    return request.param()


@pytest.mark.parametrize("create_app", [flask_app, pyramid_app])
@pytest.mark.parametrize(
    "get_user_id", [lambda: 1 / 0, lambda: b"1", lambda: Exception()]
)
def test_exception_in_logger_doesnt_propagate(create_app, get_user_id, caplog):
    """Test that an error when fetching the user id or serializing it doesn't crash the app."""
    response = create_app(get_user_id).get("/test")
    assert response.status_code == 200
    assert not caplog.records


@pytest.mark.parametrize("create_app", [flask_app, pyramid_app])
def test_does_not_log_without_user(create_app, caplog):
    create_app(lambda: None).get("/test")
    assert not caplog.records


def test_does_not_log_when_hitting_static(app, caplog):
    app.get("/static/1")
    app.get("/static/2")
    assert not caplog.records


@pytest.mark.parametrize(
    "query,expected",
    [
        ("", {}),
        ("?name=j%C3%B3h%C3%B1&age=33", {"name": "jóhñ", "age": "33"}),
        ("?role=teacher&role=student", {"role": ["teacher", "student"]}),
    ],
)
def test_get_request(app, caplog, query, expected):
    app.get("/test{}".format(query))

    match = re.match(
        r"\[FullRequestInfo\] /test user=john args=(.*)", caplog.records[0].message
    )
    assert match and json.loads(match.group(1)) == expected


def test_when_error_request(app, caplog):
    try:
        app.get("/error", expect_errors=True)
    except Exception:
        pass
    assert caplog.records[0].message == "[FullRequestInfo] /error user=john args={}"
